#!/usr/bin/perl
# Show a form for registering a xen host, and editing config settings

require './server-manager-lib.pl';
&can_action_error(undef, "global");
&ReadParse();
&ui_print_header(undef, $in{'new'} ? $text{'xen_create'} :
		        $module_info{'onehost'} ? $text{'xen_editone'} :
						  $text{'xen_edit'}, "",
		 $in{'new'} ? "createxen" : "editxen");
@allservers = &list_managed_servers();
if (!$in{'new'}) {
	($xen) = grep { $_->{'id'} eq $in{'id'} } @allservers;
	$xen || &error($text{'xen_egone'});
	}
else {
	$xen = { };
	}

print &ui_form_start("save_xen.cgi", "post");
print &ui_hidden("new", $in{'new'});
print &ui_hidden_table_start($text{'xen_header'}, "width=100%", 2, "main", 1);

# System to register
if ($in{'new'}) {
	@canservers = grep { &can_host_xen($_) &&
			     !$_->{'xen_host'} } @allservers;
	print &ui_table_row($text{'xen_host'},
		&ui_select("id", undef,
			[ map { [ $_->{'id'}, $_->{'host'} ] } @canservers ]));
	}
else {
	print &ui_hidden("id", $in{'id'});
	if (!$module_info{'onehost'}) {
		print &ui_table_row($text{'xen_host'},
				    $xen->{'host'});
		}
	}

if (!$module_info{'onehost'}) {
	# Location group
	&show_host_location_field($xen);
	}

# Xen directory
print &ui_table_row($text{'xen_dir'},
	&ui_textbox("dir", $xen->{'xen_dir'} || "/xen", 60));

# Find volume groups on host, if we can
@pvs = ( );
if (!$in{'new'} && $xen->{'status'} ne 'down' &&
    &remote_foreign_check($xen, "lvm")) {
	$last_remote_error = undef;
	&remote_foreign_require($xen, "lvm", "lvm-lib.pl");
	if (!$last_remote_error) {
		@pvs = &remote_foreign_call($xen, "lvm",
					    "list_volume_groups");
		}
	}
if (@pvs) {
	$lvfield = &ui_select("lvm", $xen->{'xen_vg'},
			      [ map { $_->{'name'} } @pvs ], 1, 0,
			      $xen->{'xen_vg'} ? 1 : 0);
	}
else {
	$lvfield = &ui_textbox("lvm", $xen->{'xen_vg'}, 40);
	}

# LVM volume group for disks
print &ui_table_row($text{'xen_lvm'},
	&ui_radio("lvm_def", $xen->{'xen_vg'} ? 0 : 1,
		  [ [ 1, $text{'xen_nolvm'}."<br>" ],
		    [ 0, $text{'xen_lvmvg'}." ".$lvfield ] ]));

# DNS domain name to add under (if we host any)
@zones = &list_bind_zones();
$zonename = $xen->{'xen_zone'} || $config{'xen_zone'};
$link = " ".&bind_module_link();
if (@zones) {
	print &ui_table_row($text{'xen_domain'},
		&ui_select("zone", $zonename,
		    [ map { [ $_->{'name'} ] } @zones ]).$link);
	}
else {
	print &ui_table_row($text{'xen_domain2'},
		&ui_textbox("zone", $zonename, 30).$link);
	}

# Xen file driver type
$xen->{'xen_filetype'} = 'file' if (!$xen->{'xen_filetype'} && !$in{'new'});
print &ui_table_row($text{'xen_filetype'},
	&ui_radio("filetype", $xen->{'xen_filetype'},
		  [ $in{'new'} ? ( [ "", $text{'xen_ft_auto'} ] ) : ( ),
		    [ "file", $text{'xen_ft_file'} ],
		    [ "tap:aio", $text{'xen_ft_aio'} ] ]));

if (!$module_info{'onehost'}) {
	# Live migration IP and port
	print &ui_table_row($text{'xen_livehost'},
		&ui_radio("livehost_def", $xen->{'xen_livehost'} ? 0 : 1,
			  [ [ 1, $text{'xen_livehost1'}."<br>" ],
			    [ 0, $text{'xen_livehost0'} ] ])." ".
		&ui_textbox("livehost", $xen->{'xen_livehost'}, 30)." ".
		$text{'xen_liveport'}." ".
		&ui_textbox("liveport",
			    $xen->{'xen_liveport'} || $xen_migration_port, 5));
	}

# Add all existing Xen instances at same time?
if ($in{'new'}) {
	# Auto-detect?
	print &ui_table_row($text{'xen_autoadd'},
		&ui_radio("autoadd", 1,
			  [ [ 0, $text{'no'} ],
			    [ 1, $text{'xen_autoadd1'} ],
			    [ 2, $text{'xen_autoadd2'} ] ]));

	# Login type
	print &ui_table_row($text{'xen_autologin'},
		&ui_radio("autologin", 1,
			  [ [ 0, $text{'xen_autologin0'} ],
			    [ 1, $text{'xen_autologin1'} ] ]));
	}

print &ui_hidden_table_end("main");

print &ui_hidden_table_start($text{'xen_nheader'}, "width=100%", 2, "net", 0);

# IP allocation range and CIDR
if (!$in{'new'}) {
	@possifaces = &list_host_bridges($xen);
	}
if (!@possifaces) {
	# Don't know host yet, so just use fixed list
	@possifaces = ( "xenbr0", "xenbr1" );
	}
@hostifaces = split(/\s+/, $xen->{'xen_iface'});
@allocifaces = &unique(@possifaces, @hostifaces);
print &ui_table_row($text{'xen_range'},
	&show_ip_range_input($xen, "xen", 1, \@allocifaces));

# Network interfaces to use on host system
if (@possifaces > 1 || @hostifaces > 1) {
	print &ui_table_row($text{'xen_iface'},
		&ui_radio("iface_def", scalar(@hostifaces) ? 0 : 1,
			  [ [ 1, $text{'xen_iface1'} ],
			    [ 0, $text{'xen_iface0'} ] ])."<br>\n".
		&ui_select("iface", \@hostifaces, \@possifaces,
			   scalar(@possifaces), 1, 1));
	}
else {
	# No need to change setting
	print &ui_hidden("iface", $xen->{'xen_iface'});
	}

# Source resolv.conf file
print &ui_table_row($text{'xen_resolv'},
	&ui_opt_textbox("resolv", $xen->{'xen_resolv'}, 30,
			$text{'xen_resolvdef'}, $text{'xen_resolvother'})." ".
	&file_chooser_button("resolv"));

print &ui_hidden_table_end("net");

# Section for minimum resources
&show_host_floor_table($xen);

if (!$module_info{'onehost'}) {
	# Section for caching
	&show_host_images_cache_table($xen, "xen");
	}

# Section for default backup destination
&show_default_backup_dest_table($xen);

# Start of section for hosted systems
@users = grep { $_->{'xen_on'} eq $xen->{'id'} } @allservers;
if (!$in{'new'} && @users) {
	print &ui_hidden_table_start($text{'xen_header2'}, "width=100%", 2,
				     "hosts", 0);

	# Hosted systems and their disk/memory consumption
	$utable = &ui_columns_start([ $text{'xen_uhost'},
				      $text{'xen_udisk'},
				      $text{'xen_umemory'},
				      $text{'xen_ustate'} ], 100);
	$disksum = $memorysum = $upmemorysum = $vgsum = 0;
	foreach $u (sort { $a->{'host'} cmp $b->{'host'} } @users) {
		$utable .= &ui_columns_row([
			"<a href='edit_serv.cgi?id=$u->{'id'}'>".
			$u->{'host'}."</a>",
			$u->{'xen_disktotal'} ?
				&nice_size($u->{'xen_disktotal'}) :
				$text{'domains_unknown'},
			&nice_size($u->{'xen_memory'}*1024*1024),
			&describe_status($u, 1),
			]);
		if ($u->{'xen_filesystem_vg'}) {
			$vgsum += $u->{'xen_disktotal'};
			}
		else {
			$disksum += $u->{'xen_disktotal'};
			}
		$memorysum += $u->{'xen_memory'}*1024*1024;
		$upmemorysum += $u->{'xen_memory'}*1024*1024
			if ($u->{'status'} ne 'down');
		}
	$utable .= &ui_columns_end();
	print &ui_table_row(undef, $utable, 2);

	# Total disk and amount consumed
	@tds = ( "width=30%" );
	$info = &get_server_collected($xen);
	if ($info->{'disk_total'}) {
		print &ui_table_row($text{'xen_diskalloc'},
			&text('xen_diskmsg',
			      &nice_size($info->{'disk_total'}),
			      &nice_size($disksum),
			      int($disksum*100/$info->{'disk_total'})),
			1, \@tds);
		}

	# Total LVM disk and amount consumed
	if ($xen->{'xen_vg_total'}) {
		print &ui_table_row($text{'xen_vgalloc'},
			&text('xen_diskmsg',
			      &nice_size($xen->{'xen_vg_total'}),
			      &nice_size($vgsum),
			      int($vgsum*100/$xen->{'xen_vg_total'})),
			1, \@tds);
		}

	# Total memory and amount consumed
	@mem = map { $_*1024 } @{$info->{'mem'}};
	if ($mem[0]) {
		# For Xen hosts, total memory as reported by 'free' does not
		# include that assigned to Xen instances. So we add it here..
		$mem[0] += $upmemorysum;
		}
	if ($mem[0]) {
		$col = "";
		if ($memorysum > $mem[0]) {
			# Over-committed!
			$col = "color='#ff0000'";
			}
		print &ui_table_row($text{'xen_memoryalloc'},
			"<font $col>".
			&text('xen_diskmsg',
				&nice_size($mem[0]),
				&nice_size($memorysum),
				int($memorysum*100/$mem[0])).
			"</font>", 1, \@tds);
		}

	# For running systems
	if ($mem[0] && $memorysum != $upmemorysum) {
		print &ui_table_row($text{'xen_memoryalloc2'},
			&text('xen_diskmsg',
				&nice_size($mem[0]),
				&nice_size($upmemorysum),
				int($upmemorysum*100/$mem[0])), 1, \@tds);
		}

	print &ui_hidden_table_end("hosts");
	}

print &ui_form_end($in{'new'} ? [ [ undef, $text{'xen_reg'} ] ]
			      : [ [ undef, $text{'save'} ],
				  [ "delete", $text{'xen_unreg'} ] ]);

if ($module_info{'onehost'}) {
	&ui_print_footer("", $text{'index_return'});
	}
else {
	&ui_print_footer("list_xens.cgi", $text{'xens_return'});
	}

